/* ------------------------------------------------------------------------
System       : cc65 cross-development environment
Platform     : Apple IIgs or Apple IIe with VOC or CB

Bill Buckels
July 2014

Licence      : You may use this program for whatever you wish as long
			   as you agree that neither Charlie nor Bill have any
			   warranty or liability obligations whatsoever from said
			   use.
------------------------------------------------------------------------ */
#include <peekpoke.h>

#define  gswitch ((unsigned char*)0xC029)
#define  shadow  ((unsigned char*)0xC035)

/* graphics - save previous setting of gswitch */
unsigned char gsave = 0xff;
/* shadow memory - save previous setting of shadow switch */
unsigned char ssave = 0xff;

/* move a block of data from main to auxiliary memory */
#pragma optimize (push,off)
void maintoaux(unsigned src0, unsigned src1, unsigned dest0)
{

    unsigned *src  = (unsigned *)0x3c;
    unsigned *dest = (unsigned *)0x42;

    src[0] = src0;
    src[1] = src1;
    dest[0] = dest0;

    asm("sec");
    asm("jsr $c311");

}
#pragma optimize (pop)

void shgron(void)
{

    gsave = gswitch[0]; 		/* save previous gswitch settings */
	gswitch[0]= gsave | 0xc0;   /* shgr on  - set bits 6 and 7 */

	ssave = shadow[0];
	shadow[0] = ssave & 0xf7;

}

void shgroff()
{
	gswitch[0]=gsave; /* shgr off - restore previous setting */
	shadow[0] =ssave;
}


unsigned char palbuf[512], shrpal[32], scbbuf[256], scanline[160];

void clearpalette()
{
	unsigned src1 = (unsigned)&palbuf[0];
	unsigned src2 = src1 + 511;

	/* zero palette in auxiliary memory */
    memset((char *)&palbuf[0],0,512);
	maintoaux(src1,src2,0x9e00);
}

void clearscbs()
{
	unsigned src1 = (unsigned)&scbbuf[0];
	unsigned src2 = src1 + 199;

	/* zero scanline control bytes in auxiliary memory */
    memset((char *)&scbbuf[0],1,200);
    memset((char *)&scbbuf[200],0,56);
	maintoaux(src1,src2,0x9d00);

}

#pragma optimize (push,off)
void setscb(unsigned y, unsigned palidx)
{

    unsigned *src  = (unsigned *)0x3c;
    unsigned *dest = (unsigned *)0x42;

	scbbuf[y] = (unsigned char)palidx;
	src[0] = src[1] = (unsigned)&scbbuf[y];
	dest[0] = (unsigned)0x9d00 + y;

    asm("sec");
    asm("jsr $c311");

}
#pragma optimize (pop)

#pragma optimize (push,off)
void putline(unsigned y)
{

    unsigned *src  = (unsigned *)0x3c;
    unsigned *dest = (unsigned *)0x42;

    src[0] = (unsigned) &scanline[0];
    src[1] = (unsigned) src[0]+159;
	dest[0] = (unsigned) (0x2000 + (y * 160));

	asm("sec");
    asm("jsr $c311");

}
#pragma optimize (pop)


/* turn-on text mode */
void texton(void)
{
	POKE(0xc051,0);  /* text */
	POKE(0xc054,0);  /* page 1 */
}
